import { Actor } from './actor.js';
import { ValidationPool } from './validation-pool.js';
import params from '../params.js';
import { Action } from './action.js';
import { ReputationTokenContract } from './reputation-token.js';

/**
 * Purpose: Keep track of reputation holders
 */
export class Bench extends Actor {
  constructor(forum, name, scene) {
    super(name, scene);
    this.forum = forum;
    this.validationPools = new Map();
    this.voters = new Map();
    this.reputation = new ReputationTokenContract();

    this.actions = {
      createValidationPool: new Action('create validation pool', scene),
    };
  }

  listValidationPools() {
    Array.from(this.validationPools.values());
  }

  listActiveVoters() {
    return Array.from(this.voters.values()).filter((voter) => {
      const hasVoted = !!voter.dateLastVote;
      const withinThreshold = !params.activeVoterThreshold
      || new Date() - voter.dateLastVote >= params.activeVoterThreshold;
      return hasVoted && withinThreshold;
    });
  }

  getActiveReputation() {
    return this.listActiveVoters()
      .map(({ reputationPublicKey }) => this.reputation.valueOwnedBy(reputationPublicKey))
      .reduce((acc, cur) => (acc += cur), 0);
  }

  getActiveAvailableReputation() {
    return this.listActiveVoters()
      .map(({ reputationPublicKey }) => this.reputation.availableValueOwnedBy(reputationPublicKey))
      .reduce((acc, cur) => (acc += cur), 0);
  }

  async initiateValidationPool(poolOptions) {
    const validationPoolNumber = this.validationPools.size + 1;
    const name = `Pool${validationPoolNumber}`;
    const validationPool = new ValidationPool(this, this.forum, poolOptions, name, this.scene);
    this.validationPools.set(validationPool.id, validationPool);
    await this.actions.createValidationPool.log(this, validationPool);
    validationPool.activate();
    return validationPool;
  }
}
